@@ -1,7 +1,25 @@
 # All windows write only unicode to the terminal -
 # that's what blessings does, so we match it.
 
-from __future__ import unicode_literals
+
+from typing import (
+    Optional,
+    Text,
+    IO,
+    Dict,
+    Generic,
+    TypeVar,
+    Type,
+    Tuple,
+    Callable,
+    Any,
+    ByteString,
+    cast,
+    TextIO,
+    Union,
+    List,
+)
+from types import TracebackType
 
 import locale
 import logging
@@ -10,51 +28,60 @@ import sys
 
 import blessings
 
-from .formatstring import fmtstr
+from .formatstring import fmtstr, FmtStr
 from .formatstringarray import FSArray
 from .termhelpers import Cbreak
 
 logger = logging.getLogger(__name__)
 
-SCROLL_DOWN = u"\x1bD"
-FIRST_COLUMN = u"\x1b[1G"
+SCROLL_DOWN = "\x1bD"
+FIRST_COLUMN = "\x1b[1G"
 
 
-class BaseWindow(object):
+T = TypeVar("T", bound="BaseWindow")
+
+
+class BaseWindow:
     def __init__(self, out_stream=None, hide_cursor=True):
-        logger.debug('-------initializing Window object %r------' % self)
+        # type: (IO, bool) -> None
+        logger.debug("-------initializing Window object %r------" % self)
         if out_stream is None:
             out_stream = sys.__stdout__
         self.t = blessings.Terminal(stream=out_stream, force_styling=True)
         self.out_stream = out_stream
         self.hide_cursor = hide_cursor
-        self._last_lines_by_row = {}
-        self._last_rendered_width = None
-        self._last_rendered_height = None
+        self._last_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
+        self._last_rendered_width = None  # type: Optional[int]
+        self._last_rendered_height = None  # type: Optional[int]
 
     def scroll_down(self):
-        logger.debug('sending scroll down message w/ cursor on bottom line')
+        # type: () -> None
+        logger.debug("sending scroll down message w/ cursor on bottom line")
 
         # since scroll-down only moves the screen if cursor is at bottom
         with self.t.location(x=0, y=1000000):
             self.write(SCROLL_DOWN)  # TODO will blessings do this?
 
     def write(self, msg):
+        # type: (Text) -> None
         self.out_stream.write(msg)
         self.out_stream.flush()
 
     def __enter__(self):
+        # type: (T) -> T
         logger.debug("running BaseWindow.__enter__")
         if self.hide_cursor:
             self.write(self.t.hide_cursor)
         return self
 
     def __exit__(self, type, value, traceback):
+        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
         logger.debug("running BaseWindow.__exit__")
         if self.hide_cursor:
             self.write(self.t.normal_cursor)
 
     def on_terminal_size_change(self, height, width):
+        # type: (int, int) -> None
         # Changing the terminal size breaks the cache, because it
         # is unknown how the window size change affected scrolling / the cursor
         self._last_lines_by_row = {}
@@ -62,30 +89,41 @@ class BaseWindow(object):
         self._last_rendered_height = height
 
     def render_to_terminal(self, array, cursor_pos=(0, 0)):
+        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> Optional[int]
         raise NotImplementedError
 
     def get_term_hw(self):
+        # type: () -> Tuple[int, int]
         """Returns current terminal height and width"""
         return self.t.height, self.t.width
 
-    width = property(lambda self: self.t.width, None, None,
-                     "The current width of the terminal window")
-    height = property(lambda self: self.t.height, None, None,
-                      "The current height of the terminal window")
+    @property
+    def width(self):
+        # type: () -> int
+        "The current width of the terminal window"
+        return self.t.width
+
+    @property
+    def height(self):
+        # type: () -> int
+        "The current width of the terminal window"
+        return self.t.height
 
     def array_from_text(self, msg):
+        # type: (Text) -> FSArray
         """Returns a FSArray of the size of the window containing msg"""
         rows, columns = self.t.height, self.t.width
         return self.array_from_text_rc(msg, rows, columns)
 
     @classmethod
     def array_from_text_rc(cls, msg, rows, columns):
+        # type: (Text, int, int) -> FSArray
         arr = FSArray(0, columns)
         i = 0
         for c in msg:
             if i >= rows * columns:
                 return arr
-            elif c in '\r\n':
+            elif c in "\r\n":
                 i = ((i // columns) + 1) * columns - 1
             else:
                 arr[i // arr.width, i % arr.width] = [fmtstr(c)]
@@ -93,22 +131,28 @@ class BaseWindow(object):
         return arr
 
     def fmtstr_to_stdout_xform(self):
+        # type: () -> Callable[[FmtStr], Text]
         if sys.version_info[0] == 2:
-            if hasattr(self.out_stream, 'encoding'):
+            if hasattr(self.out_stream, "encoding"):
                 encoding = self.out_stream.encoding
             else:
                 encoding = locale.getpreferredencoding()
 
             def for_stdout(s):
-                return unicode(s).encode(encoding, 'replace')
+                # type: (FmtStr) -> Text
+                return unicode(s).encode(encoding, "replace")
+
         else:
+
             def for_stdout(s):
+                # type: (FmtStr) -> Text
                 return str(s)
+
         return for_stdout
 
 
 class FullscreenWindow(BaseWindow):
-    """2D-text rendering window that dissappears when its context is left
+    """2D-text rendering window that disappears when its context is left
 
     FullscreenWindow will only render arrays the size of the terminal
     or smaller, and leaves no trace on exit (like top or vim). It never
@@ -122,26 +166,30 @@ class FullscreenWindow(BaseWindow):
         Within the context of CursorAwareWindow, refrain from writing to
         its out_stream; cached writes will be inaccurate.
     """
+
     def __init__(self, out_stream=None, hide_cursor=True):
+        # type: (IO, bool) -> None
         """Constructs a FullscreenWindow
 
         Args:
             out_stream (file): Defaults to sys.__stdout__
             hide_cursor (bool): Hides cursor while in context
         """
-        BaseWindow.__init__(self, out_stream=out_stream,
-                            hide_cursor=hide_cursor)
+        BaseWindow.__init__(self, out_stream=out_stream, hide_cursor=hide_cursor)
         self.fullscreen_ctx = self.t.fullscreen()
 
     def __enter__(self):
+        # type: () -> FullscreenWindow
         self.fullscreen_ctx.__enter__()
         return BaseWindow.__enter__(self)
 
     def __exit__(self, type, value, traceback):
+        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
         self.fullscreen_ctx.__exit__(type, value, traceback)
         BaseWindow.__exit__(self, type, value, traceback)
 
     def render_to_terminal(self, array, cursor_pos=(0, 0)):
+        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> None
         """Renders array to terminal and places (0-indexed) cursor
 
         Args:
@@ -164,14 +212,13 @@ class FullscreenWindow(BaseWindow):
         for_stdout = self.fmtstr_to_stdout_xform()
         if not self.hide_cursor:
             self.write(self.t.hide_cursor)
-        if (height != self._last_rendered_height or
-                width != self._last_rendered_width):
+        if height != self._last_rendered_height or width != self._last_rendered_width:
             self.on_terminal_size_change(height, width)
 
-        current_lines_by_row = {}
+        current_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
         rows = list(range(height))
-        rows_for_use = rows[:len(array)]
-        rest_of_rows = rows[len(array):]
+        rows_for_use = rows[: len(array)]
+        rest_of_rows = rows[len(array) :]
 
         # rows which we have content for and don't require scrolling
         for row, line in zip(rows_for_use, array):
@@ -192,12 +239,8 @@ class FullscreenWindow(BaseWindow):
             self.write(self.t.clear_bol)
             current_lines_by_row[row] = None
 
-        logger.debug(
-            'lines in last lines by row: %r' % self._last_lines_by_row.keys()
-        )
-        logger.debug(
-            'lines in current lines by row: %r' % current_lines_by_row.keys()
-        )
+        logger.debug("lines in last lines by row: %r" % self._last_lines_by_row.keys())
+        logger.debug("lines in current lines by row: %r" % current_lines_by_row.keys())
         self.write(self.t.move(*cursor_pos))
         self._last_lines_by_row = current_lines_by_row
         if not self.hide_cursor:
@@ -218,9 +261,16 @@ class CursorAwareWindow(BaseWindow):
         cursor depends on cursor not having moved since the last render.
         Only use the render_to_terminal interface for moving the cursor.
     """
-    def __init__(self, out_stream=None, in_stream=None,
-                 keep_last_line=False, hide_cursor=True,
-                 extra_bytes_callback=None):
+
+    def __init__(
+        self,
+        out_stream=None,
+        in_stream=None,
+        keep_last_line=False,
+        hide_cursor=True,
+        extra_bytes_callback=None,
+    ):
+        # type: (IO, IO, bool, bool, Callable[[ByteString], None]) -> None
         """Constructs a CursorAwareWindow
 
         Args:
@@ -230,16 +280,15 @@ class CursorAwareWindow(BaseWindow):
                 on leaving context
             hide_cursor (bool): Hides cursor while in context
             extra_bytes_callback (f(bytes) -> None): Will be called with extra
-                bytes inadvertantly read in get_cursor_position(). If not
+                bytes inadvertently read in get_cursor_position(). If not
                 provided, a ValueError will be raised when this occurs.
         """
-        BaseWindow.__init__(self, out_stream=out_stream,
-                            hide_cursor=hide_cursor)
+        BaseWindow.__init__(self, out_stream=out_stream, hide_cursor=hide_cursor)
         if in_stream is None:
             in_stream = sys.__stdin__
         self.in_stream = in_stream
-        self._last_cursor_column = None
-        self._last_cursor_row = None
+        self._last_cursor_column = None  # type: Optional[int]
+        self._last_cursor_row = None  # type: Optional[int]
         self.keep_last_line = keep_last_line
         self.cbreak = Cbreak(self.in_stream)
         self.extra_bytes_callback = extra_bytes_callback
@@ -251,13 +300,15 @@ class CursorAwareWindow(BaseWindow):
         self.in_get_cursor_diff = False
 
     def __enter__(self):
+        # type: () -> CursorAwareWindow
         self.cbreak.__enter__()
         self.top_usable_row, _ = self.get_cursor_position()
         self._orig_top_usable_row = self.top_usable_row
-        logger.debug('initial top_usable_row: %d' % self.top_usable_row)
+        logger.debug("initial top_usable_row: %d" % self.top_usable_row)
         return BaseWindow.__enter__(self)
 
     def __exit__(self, type, value, traceback):
+        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
         if self.keep_last_line:
             # just moves cursor down if not on last line
             self.write(SCROLL_DOWN)
@@ -269,55 +320,70 @@ class CursorAwareWindow(BaseWindow):
         BaseWindow.__exit__(self, type, value, traceback)
 
     def get_cursor_position(self):
+        # type: () -> Tuple[int, int]
         """Returns the terminal (row, column) of the cursor
 
         0-indexed, like blessings cursor positions"""
         # TODO would this be cleaner as a parameter?
         in_stream = self.in_stream
 
-        query_cursor_position = u"\x1b[6n"
+        query_cursor_position = "\x1b[6n"
         self.write(query_cursor_position)
 
         def retrying_read():
+            # type: () -> str
             while True:
                 try:
                     c = in_stream.read(1)
-                    if c == '':
-                        raise ValueError("Stream should be blocking - should't"
-                                         " return ''. Returned %r so far",
-                                         (resp,))
+                    if c == "":
+                        raise ValueError(
+                            "Stream should be blocking - shouldn't"
+                            " return ''. Returned %r so far",
+                            (resp,),
+                        )
                     return c
-                except IOError:
-                    raise ValueError(
-                        'cursor get pos response read interrupted'
-                    )
-                    # find out if this ever really happens - if so, continue
+                except OSError:
+                    # apparently sometimes this happens: the only documented
+                    # case is Terminal on a Ubuntu 17.10 VM on osx 10.13.
+                    # see issue #732
+                    logger.info("stdin.read(1) that should never error just errored.")
+                    continue
 
-        resp = ''
+        resp = ""
         while True:
             c = retrying_read()
             resp += c
-            m = re.search('(?P<extra>.*)'
-                          '(?P<CSI>\x1b\[|\x9b)'
-                          '(?P<row>\\d+);(?P<column>\\d+)R', resp, re.DOTALL)
+            m = re.search(
+                r"(?P<extra>.*)"
+                r"(?P<CSI>\x1b\[|\x9b)"
+                r"(?P<row>\d+);(?P<column>\d+)R",
+                resp,
+                re.DOTALL,
+            )
             if m:
-                row = int(m.groupdict()['row'])
-                col = int(m.groupdict()['column'])
-                extra = m.groupdict()['extra']
+                row = int(m.groupdict()["row"])
+                col = int(m.groupdict()["column"])
+                extra = m.groupdict()["extra"]
                 if extra:
                     if self.extra_bytes_callback:
                         self.extra_bytes_callback(
-                            extra.encode(in_stream.encoding)
+                            # TODO how do we know that this works?
+                            extra.encode(cast(TextIO, in_stream).encoding)
                         )
                     else:
-                        raise ValueError(("Bytes preceding cursor position "
-                                          "query response thrown out:\n%r\n"
-                                          "Pass an extra_bytes_callback to "
-                                          "CursorAwareWindow to prevent this")
-                                         % (extra,))
+                        raise ValueError(
+                            (
+                                "Bytes preceding cursor position "
+                                "query response thrown out:\n%r\n"
+                                "Pass an extra_bytes_callback to "
+                                "CursorAwareWindow to prevent this"
+                            )
+                            % (extra,)
+                        )
                 return (row - 1, col - 1)
 
     def get_cursor_vertical_diff(self):
+        # type: () -> int
         """Returns the how far down the cursor moved since last render.
 
         Note:
@@ -347,6 +413,7 @@ class CursorAwareWindow(BaseWindow):
                 return cursor_dy
 
     def _get_cursor_vertical_diff_once(self):
+        # type: () -> int
         """Returns the how far down the cursor moved."""
         old_top_usable_row = self.top_usable_row
         row, col = self.get_cursor_position()
@@ -354,20 +421,24 @@ class CursorAwareWindow(BaseWindow):
             cursor_dy = 0
         else:
             cursor_dy = row - self._last_cursor_row
-            logger.info('cursor moved %d lines down' % cursor_dy)
+            logger.info("cursor moved %d lines down" % cursor_dy)
             while self.top_usable_row > -1 and cursor_dy > 0:
                 self.top_usable_row += 1
                 cursor_dy -= 1
             while self.top_usable_row > 1 and cursor_dy < 0:
                 self.top_usable_row -= 1
                 cursor_dy += 1
-        logger.info('top usable row changed from %d to %d', old_top_usable_row,
-                    self.top_usable_row)
-        logger.info('returning cursor dy of %d from curtsies' % cursor_dy)
+        logger.info(
+            "top usable row changed from %d to %d",
+            old_top_usable_row,
+            self.top_usable_row,
+        )
+        logger.info("returning cursor dy of %d from curtsies" % cursor_dy)
         self._last_cursor_row = row
         return cursor_dy
 
     def render_to_terminal(self, array, cursor_pos=(0, 0)):
+        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> int
         """Renders array to terminal, returns the number of lines scrolled offscreen
 
         Returns:
@@ -394,11 +465,10 @@ class CursorAwareWindow(BaseWindow):
 
         # TODO race condition here?
         height, width = self.t.height, self.t.width
-        if (height != self._last_rendered_height or
-                width != self._last_rendered_width):
+        if height != self._last_rendered_height or width != self._last_rendered_width:
             self.on_terminal_size_change(height, width)
 
-        current_lines_by_row = {}
+        current_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
         rows_for_use = list(range(self.top_usable_row, height))
 
         # rows which we have content for and don't require scrolling
@@ -433,28 +503,20 @@ class CursorAwareWindow(BaseWindow):
                 self.top_usable_row -= 1
             else:
                 offscreen_scrolls += 1
-            current_lines_by_row = dict(
-                (k - 1, v) for k, v in current_lines_by_row.items()
-            )
-            logger.debug('new top_usable_row: %d' % self.top_usable_row)
+            current_lines_by_row = {k - 1: v for k, v in current_lines_by_row.items()}
+            logger.debug("new top_usable_row: %d" % self.top_usable_row)
             # since scrolling moves the cursor
             self.write(self.t.move(height - 1, 0))
             self.write(for_stdout(line))
             current_lines_by_row[height - 1] = line
 
-        logger.debug(
-            'lines in last lines by row: %r' % self._last_lines_by_row.keys()
-        )
-        logger.debug(
-            'lines in current lines by row: %r' % current_lines_by_row.keys()
-        )
+        logger.debug("lines in last lines by row: %r" % self._last_lines_by_row.keys())
+        logger.debug("lines in current lines by row: %r" % current_lines_by_row.keys())
         self._last_cursor_row = max(
             0, cursor_pos[0] - offscreen_scrolls + self.top_usable_row
         )
         self._last_cursor_column = cursor_pos[1]
-        self.write(
-            self.t.move(self._last_cursor_row, self._last_cursor_column)
-        )
+        self.write(self.t.move(self._last_cursor_row, self._last_cursor_column))
         self._last_lines_by_row = current_lines_by_row
         if not self.hide_cursor:
             self.write(self.t.normal_cursor)
@@ -462,19 +524,23 @@ class CursorAwareWindow(BaseWindow):
 
 
 def demo():
-    handler = logging.FileHandler(filename='display.log')
+    # type: () -> None
+    handler = logging.FileHandler(filename="display.log")
     logging.getLogger(__name__).setLevel(logging.DEBUG)
     logging.getLogger(__name__).addHandler(handler)
     from . import input
+
     with FullscreenWindow(sys.stdout) as w:
         with input.Input(sys.stdin) as input_generator:
             rows, columns = w.t.height, w.t.width
-            while True:
-                c = input_generator.next()
+            for c in input_generator:
+                assert isinstance(c, Text)
                 if c == "":
                     sys.exit()  # same as raise SystemExit()
                 elif c == "h":
-                    a = w.array_from_text("a for small array")
+                    a = w.array_from_text(
+                        "a for small array"
+                    )  # type: Union[List[FmtStr], FSArray]
                 elif c == "a":
                     a = [fmtstr(c * columns) for _ in range(rows)]
                 elif c == "s":
@@ -489,8 +555,9 @@ def demo():
                     a = [fmtstr(c * columns) for _ in range(1)]
                 elif c == "e":
                     a = [fmtstr(c * columns) for _ in range(1)]
-                elif c == '\x0c':  # ctrl-L
-                    [w.write('\n') for _ in range(rows)]
+                elif c == "\x0c":  # ctrl-L
+                    for _ in range(rows):
+                        w.write("\n")
                     continue
                 else:
                     a = w.array_from_text("unknown command")
@@ -498,17 +565,17 @@ def demo():
 
 
 def main():
-    handler = logging.FileHandler(filename='display.log', level=logging.DEBUG)
+    # type: () -> None
+    handler = logging.FileHandler(filename="display.log")
     logging.getLogger(__name__).setLevel(logging.DEBUG)
     logging.getLogger(__name__).addHandler(handler)
-    print('this should be just off-screen')
+    print("this should be just off-screen")
     w = FullscreenWindow(sys.stdout)
     rows, columns = w.t.height, w.t.width
     with w:
-        a = [fmtstr(
-            (('.row%r.' % (row,)) * rows)[:columns]
-        ) for row in range(rows)]
+        a = [fmtstr(((f".row{row!r}.") * rows)[:columns]) for row in range(rows)]
         w.render_to_terminal(a)
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     demo()
