@@ -1,5 +1,3 @@
-from __future__ import unicode_literals, division
-
 import time
 
 from curtsies import FullscreenWindow, Input, FSArray
@@ -9,7 +7,7 @@ import curtsies.events
 class Frame(curtsies.events.ScheduledEvent):
     pass
 
-class World(object):
+class World:
     def __init__(self):
         self.s = 'Hello'
     def tick(self):
@@ -35,7 +33,7 @@ def realtime(fps=15):
                 while when < time.time():
                     when += dt
                 schedule_next_frame(when)
-            elif e == u'<ESC>':
+            elif e == '<ESC>':
                 break
             else:
                 world.process_event(e)
