@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
 import sys
 import unittest
 from curtsies.formatstring import (
@@ -22,32 +20,14 @@ from curtsies.fmtfuncs import (
     bold,
 )
 from curtsies.termformatconstants import FG_COLORS
-from curtsies.formatstringarray import fsarray, FSArray, FormatStringTest
+from curtsies.formatstringarray import fsarray, FSArray
 
-try:
-    from unittest import skip
-except ImportError:
-
-    def skip(f):
-        return lambda self: None
-
-
-PY2 = sys.version_info[0] == 2
-
-try:
-    unicode = unicode
-except:
-    unicode = str
+from unittest import skip
 
 
 def repr_without_leading_u(s):
-    assert isinstance(s, type(""))
-    if PY2:
-        r = repr(s)
-        assert r[0] == "u"
-        return r[1:]
-    else:
-        return repr(s)
+    assert isinstance(s, str)
+    return repr(s)
 
 
 class TestFmtStrInitialization(unittest.TestCase):
@@ -367,7 +347,7 @@ class TestSlicing(unittest.TestCase):
         self.assertEqual(s[1:], fmtstr("mp") + " ")
         self.assertEqual(blue("a\nb")[0:1], blue("a"))
 
-        # considering changing behavior so that this doens't work
+        # considering changing behavior so that this doesn't work
         # self.assertEqual(fmtstr('Hi!', 'blue')[15:18], fmtstr('', 'blue'))
 
 
@@ -383,19 +363,16 @@ class TestComposition(unittest.TestCase):
 class TestUnicode(unittest.TestCase):
     def test_output_type(self):
         self.assertEqual(type(str(fmtstr("hello", "blue"))), str)
-        self.assertEqual(type(unicode(fmtstr("hello", "blue"))), unicode)
 
     def test_normal_chars(self):
         fmtstr("a", "blue")
         str(fmtstr("a", "blue"))
-        unicode(fmtstr("a", "blue"))
         self.assertTrue(True)
 
     def test_funny_chars(self):
         fmtstr("⁇", "blue")
         str(Chunk("⁇", {"fg": "blue"}))
         str(fmtstr("⁇", "blue"))
-        unicode(fmtstr("⁇", "blue"))
         self.assertTrue(True)
 
     def test_right_sequence_in_py3(self):
@@ -528,10 +505,7 @@ class TestWidthHelpers(unittest.TestCase):
 class TestChunk(unittest.TestCase):
     def test_repr(self):
         c = Chunk("a", {"fg": 32})
-        if PY2:
-            self.assertEqual(repr(c), """Chunk(u'a', {'fg': 32})""")
-        else:
-            self.assertEqual(repr(c), """Chunk('a', {'fg': 32})""")
+        self.assertEqual(repr(c), """Chunk('a', {'fg': 32})""")
 
 
 class TestChunkSplitter(unittest.TestCase):
@@ -584,10 +558,11 @@ class TestFSArray(unittest.TestCase):
     def test_assignment_working(self):
         t = FSArray(10, 10)
         t[2, 2] = "a"
+        # TODO: is this supposed to check something?
         t[2, 2] == "a"
 
     def test_normalize_slice(self):
-        class SliceBuilder(object):
+        class SliceBuilder:
             def __getitem__(self, slice):
                 return slice
 
@@ -602,6 +577,43 @@ class TestFSArray(unittest.TestCase):
         a[2:-2, 2:-2] = fsarray(["asdf", "zxcv"])
 
 
+class FormatStringTest(unittest.TestCase):
+    def assertFSArraysEqual(self, a, b):
+        # type: (FSArray, FSArray) -> None
+        self.assertIsInstance(a, FSArray)
+        self.assertIsInstance(b, FSArray)
+        self.assertEqual(
+            (a.width, a.height),
+            (b.width, b.height),
+            f"fsarray dimensions do not match: {a.shape} {b.shape}",
+        )
+        for i, (a_row, b_row) in enumerate(zip(a, b)):
+            self.assertEqual(
+                a_row,
+                b_row,
+                "FSArrays differ first on line {}:\n{}".format(i, FSArray.diff(a, b)),
+            )
+
+    def assertFSArraysEqualIgnoringFormatting(self, a, b):
+        # type: (FSArray, FSArray) -> None
+        """Also accepts arrays of strings"""
+        self.assertEqual(
+            len(a),
+            len(b),
+            "fsarray heights do not match: %s %s \n%s \n%s"
+            % (len(a), len(b), simple_format(a), simple_format(b)),
+        )
+        for i, (a_row, b_row) in enumerate(zip(a, b)):
+            a_row = a_row.s if isinstance(a_row, FmtStr) else a_row
+            b_row = b_row.s if isinstance(b_row, FmtStr) else b_row
+            self.assertEqual(
+                a_row,
+                b_row,
+                "FSArrays differ first on line %s:\n%s"
+                % (i, FSArray.diff(a, b, ignore_formatting=True)),
+            )
+
+
 class TestFSArrayWithDiff(FormatStringTest):
     def test_diff_testing(self):
         a = fsarray(["abc", "def"])
@@ -622,6 +634,4 @@ class TestFSArrayWithDiff(FormatStringTest):
 
 
 if __name__ == "__main__":
-    import fmtstr.fmtstr
-
     unittest.main()
